using System;
using System.Windows.Forms;
using OutlookPanel;

namespace TXLooker.WorkItemExtention.Core
{
    /// <summary>
    /// Helper class to catch WM_SIZE messages via subclassing
    /// </summary>
    sealed class SubclassedWindow : NativeWindow
    {
        /// <summary>
        /// Size of subclassed window has changed
        /// </summary>
        public event EventHandler SizeChanged;

        /// <summary>
        /// Window procedure
        /// </summary>
        /// <param name="m"></param>
        protected override void WndProc(ref Message m)
        {
            base.WndProc(ref m);

            if (m.Msg == (int)SafeNativeMethods.WindowsMessages.WM_SIZE)
                OnSizeChanged();
        }

        /// <summary>
        /// Raise SizeChanged event
        /// </summary>
        private void OnSizeChanged()
        {
            if (SizeChanged != null)
                SizeChanged(this, null);
        }
    }
}