using System;
using System.Collections.Generic;
using System.Text;

namespace TXLooker.WorkItemExtention.UI
{
    public static class StringFormating
    {
        private static string RemoveWordsBefore(string str, params string[] endString)
        {
            StringBuilder stringBuilder = new StringBuilder(str);
            int length = 0;
            foreach (string s in endString)
            {
                length += s.Length + 1;
            }
            stringBuilder.Remove(0, length);
            return stringBuilder.ToString();
        }

        public static string RemoveStringFromString(string sourceString, string removeString)
        {
            if (!sourceString.Contains(removeString))
                return sourceString;
            return sourceString.Replace(sourceString, removeString);
        }

        public static string RemoveFromQueryAfter(string sourceString, string removeString)
        {
            if (!sourceString.Contains(removeString))
                return sourceString;

            string[] withoutSpaces = sourceString.Split(' ');
            bool beginAppend = true;
            string newString = string.Empty;

            foreach (string s in withoutSpaces)
            {
                if (s == removeString)
                    beginAppend = false;

                if (beginAppend)
                    newString += s + " ";
            }

            return newString;
        }

        public static string RemoveFromQueryBefore(string sourceString, bool removeWithWords, params string[] endString)
        {
            //===========================EXCEPTION====================================================
            if (endString.Length > 3)
                throw new ArgumentOutOfRangeException("Params of endString cannot be more then 3");
            //========================================================================================

            bool beginAppend = false;
            string[] withoutSpaces = sourceString.Split(' ');
            string newString = string.Empty;

            switch (endString.Length)
            {
                case 1:
                    foreach (string s in withoutSpaces)
                    {
                        if (s == endString[0])
                        {
                            beginAppend = true;
                        }
                        if (beginAppend)
                        {
                            newString += s + " ";
                        }
                    }
                    break;
                case 2:
                    foreach (string s in withoutSpaces)
                    {
                        if (s == endString[0] || s == endString[1])
                        {
                            beginAppend = true;
                        }
                        if (beginAppend)
                        {
                            newString += s + " ";
                        }
                    }
                    break;
                case 3:
                    foreach (string s in withoutSpaces)
                    {
                        if (s == endString[0] || s == endString[1] || s == endString[2])
                        {
                            beginAppend = true;
                        }
                        if (beginAppend)
                        {
                            newString += s + " ";
                        }
                    }
                    break;
                default: throw new Exception();

            }

            if (removeWithWords)
                return RemoveWordsBefore(newString, endString);
            else
                return newString;
        }

        public static string GetStringFromArray(string[] arr)
        {
            string newString = string.Empty;
            foreach (string str in arr)
            {
                newString += str + " ";
            }

            return newString;
        }



        public static bool StringConsistString(string sourceString, string stringFind)
        {
            if (sourceString == string.Empty)
            {
                throw new ArgumentNullException("sourceString is empty");
            }
            if (stringFind.Length > sourceString.Length)
            {
                throw new NotImplementedException("stringfind.Length > sourceString.Length");
            }

            char[] sourceChars = sourceString.ToCharArray();
            char[] stringFindChars = stringFind.ToCharArray();

            if (sourceChars.Length > 2 || stringFindChars.Length > 2)
                throw new Exception();

            for (int i = 0; i < sourceChars.Length; i++)
            {
                if (sourceChars[i] == stringFindChars[0] && sourceChars[i + 1] == stringFindChars[1])
                {
                    for (int j = 0; j < stringFind.Length; j++)
                    {
                        try
                        {
                            if (sourceChars[i + j] == stringFindChars[j])
                                continue;
                            else
                                return false;
                        }
                        catch
                        {
                            return false;
                        }
                    }
                }
            }
            return true;
        }

        public static string GetStringFromChars(char[] chars)
        {
            if (chars.Length == 0)
                throw new ArgumentNullException("Array is null");

            string returnString = string.Empty;
            foreach (char ch in chars)
            {
                returnString += ch.ToString();
            }

            return returnString;
        }

        /// <summary>
        ///        ,   
        /// </summary>
        /// <param name="chars"></param>
        /// <returns></returns>
        public static string DeleteSpaces(char[] chars)
        {
            try
            {
                if (chars.Length < 0)
                    return string.Empty;


                List<char> listChar = new List<char>();
                for (int i = 0; i < chars.Length; i++)
                {
                    listChar.Add(chars[i]);
                }

                if (listChar[0] == ' ')
                {
                    listChar.RemoveAt(0);
                }

                int j = 0;
                foreach (char ch in listChar)
                    j++;

                if (listChar[j - 1] == ' ')
                {
                    listChar.RemoveAt(j - 1);
                }

                return GetStringFromChars(listChar.ToArray());
            }
            catch
            {
                return GetStringFromChars(chars);
            }

        }
        /// <summary>
        ///        ,   
        /// </summary>
        /// <param name="str"></param>
        /// <returns></returns>
        public static string DeleteSpaces(string str)
        {
            return DeleteSpaces(str.ToCharArray());
        }

        public static string DeleteBrackets(string str)
        {
            return DeleteBrackets(str.ToCharArray());
        }

        /// <summary>
        ///       ,   
        /// </summary>
        /// <param name="chars"></param>
        /// <returns></returns>
        public static string DeleteBrackets(char[] chars)
        {
            if (chars.Length < 0)
                return string.Empty;

            List<char> listChar = new List<char>();
            for (int i = 0; i < chars.Length; i++)
            {
                listChar.Add(chars[i]);
            }

            if (listChar[0] == '(' && listChar[chars.Length - 1] == ')')
            {
                listChar.RemoveAt(chars.Length - 1);
                listChar.RemoveAt(0);
            }

            return GetStringFromChars(listChar.ToArray());
        }
    }
}
