namespace TXLooker.WorkItemExtention.WIQL
{
    using System;
    using System.Reflection;

    public class NodeEverOperator : Node
    {
        private Node m_child;

        public NodeEverOperator() : base(NodeType.Ever)
        {
        }

        public NodeEverOperator(Node node) : base(NodeType.Ever)
        {
            this[0] = node;
        }

        public override void Bind(IExternal e, NodeTableName tableContext, NodeFieldName fieldContext)
        {
            this.Value.Bind(e, tableContext, fieldContext);
            Tools.EnsureSyntax(this.Value.DataType == DataType.Bool, SyntaxError.ExpectingBoolean, this.Value);
            base.Bind(e, tableContext, fieldContext);
        }

        public override Node Optimize(IExternal e, NodeTableName tableContext, NodeFieldName fieldContext)
        {
            this.Value = this.Value.Optimize(e, tableContext, fieldContext);
            if (this.Value.NodeType == NodeType.BoolConst)
            {
                return this.Value;
            }
            if (this.Value.NodeType == NodeType.Ever)
            {
                return ((NodeEverOperator) this.Value).Value;
            }
            return base.Optimize(e, tableContext, fieldContext);
        }

        public override string ToString()
        {
            return ("ever " + base.JoinChildren(""));
        }

        public override int Count
        {
            get
            {
                return 1;
            }
        }

        public override DataType DataType
        {
            get
            {
                return DataType.Bool;
            }
        }

        public override bool IsConst
        {
            get
            {
                return this.m_child.IsConst;
            }
        }

        public override bool IsScalar
        {
            get
            {
                return false;
            }
        }

        public override Node this[int i]
        {
            get
            {
                return this.m_child;
            }
            set
            {
                this.m_child = value;
            }
        }

        public override Priority Priority
        {
            get
            {
                return Priority.UnaryBoolOperator;
            }
        }

        public Node Value
        {
            get
            {
                return this.m_child;
            }
            set
            {
                this.m_child = value;
            }
        }
    }
}

