using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.IO;
using System.Reflection;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using System.Xml.Serialization;
using Extensibility;
using Microsoft.Office.Core;
using Microsoft.Office.Interop.Outlook;
using Microsoft.TeamFoundation.Server;
using Microsoft.Win32;
using TeamXpandLicence.ObjectExchange;
using TXLooker.Common;
using TXLooker.Common.EventHandling;
using TXLooker.Common.UI;
using TXLooker.Core;
using TXLooker.Licence;
using TXLooker.License;
using TXLooker.Properties;
using TXLooker.UI;
using Application = Microsoft.Office.Interop.Outlook.Application;
using Exception = System.Exception;
using MAPIFolder = Microsoft.Office.Interop.Outlook.MAPIFolder;

namespace TXLooker
{
	#region Read me for Add-in installation and setup information.
	// When run, the Add-in wizard prepared the registry for the Add-in.
	// At a later time, if the Add-in becomes unavailable for reasons such as:
	//   1) You moved this project to a computer other than which is was originally created on.
	//   2) You chose 'Yes' when presented with a message asking if you wish to remove the Add-in.
	//   3) Registry corruption.
	// you will need to re-register the Add-in by building the TeamXpandSetup project, 
	// right click the project in the Solution Explorer, then choose install.
	#endregion

    #region Comments
    //procedure OnConnection(
    //   const HostApp: IDispatch;  
    //   ext_ConnectMode: Integer;
    //   const AddInInst: IDispatch;
    //   var custom: PSafeArray
    // ); safecall;  

    //    .       ,      ,     .. ...

    //,  : 
    //HostApp 	ff
    //   IDispatch  .           ,       

    //ext_ConnectMode 	
    //,   ,     : 

    //ext_cm_AfterStartup           Office 

    //ext_cm_External       

    //ext_cm_Startup           Office 

    //ext_cm_CommandLine     Office 2000 



    //AddinInst 	
    //   IDispatch     

    //Custom 	
    //     Microsoft Office   

    //procedure OnDisconnection(ext_DisconnectMode: Integer;
    //   var custom: PSafeArray); safecall;  

    //    .              COM-. 

    // ext_DisconnectMode     COM-     : 

    //ext_dm_HostShutdown    ; 

    //ext_dm_UserClosed     . 
    //procedure OnAddInsUpdate(var custom: PSafeArray); safecall;  

    //      . COM-         ,       . 
    //procedure OnStartupComplete(var custom: PSafeArray); safecall; 

    //    .   ,        ,      OnStartupComplete,      . 
    //procedure BeginShutdown(var custom: PSafeArray); safecall; 

    //     ,       - .

    #endregion

    [Guid("96A55A5A-C7E9-4df3-A3EB-D82C47F6837B"), ProgId("TXLooker.Connect")]
    public class Connect : IDTExtensibility2, IRibbonExtensibility
    {
        // Fields
        private string m_configFilePath;
        private TeamXpandConfigs m_configs;
        private Dictionary<UIControlsType, IList<CommandBarControl>> m_controlsMAP = new Dictionary<UIControlsType, IList<CommandBarControl>>();
        private Explorers m_expl;
        private ExtentionManager m_extentionManager;
        private ExtentionDataProvider m_extProvider;
        private static IntPtr m_hWnd;
        private NameSpace m_Session;
        private Folder protectedFolder;
        private string storeID;

        private EventSink eventSink = new EventSink();

        // Methods
        private void Application_FolderContextMenuDisplay(CommandBar CommandBar, MAPIFolder Folder)
        {
            OlProviderWrapperBase providerWrapperByFolder = OlProviderWrapperBase.GetProviderWrapperByFolder(Folder);
            if (providerWrapperByFolder != null)
            {
                providerWrapperByFolder.InitContextMenu(CommandBar);
            }
        }

        /// <summary>
        /// This handler is being called from Registration window
        /// </summary>
        private void CreateLicenceEMail()
        {
            MailItem item = 
                    (MailItem)Outlook
                    .Session
                    .GetDefaultFolder(OlDefaultFolders.olFolderOutbox)
                    .Items
                    .Add(OlItemType.olMailItem);

            item.To = Resources.LICENSE_E_MAIL;
            item.Display(false);
        }

        private void FolderProtectionActivate()
        {
            foreach (Explorer explorer in Outlook.Explorers)
            {
                FolderProtectionActivate(explorer);
            }
        }

        private void FolderProtectionActivate(Explorer explorer)
        {
            if (explorer != null)
            {
                try
                {
                    explorer.BeforeFolderSwitch += UnEraseFolders_BeforeFolderSwitch;
                }
                catch (System.Exception exception)
                {
                    LogManager.WriteMessageToCustomLog("Folders protecion isn't activated: " + exception.ToString());
                }
            }
        }

        private bool GetLicenceData()
        {
            LogManager.WriteMessageToCustomLog("Checking licence");

            try
            {
                ILicenceCheck check = new NullLicenseCheck(String.Empty);
                check.CheckLicence();

                ApplicationGlobal.LicenceDataProvider.LicenceType = check.LicenceType;
                ApplicationGlobal.LicenceDataProvider.DaysOff = check.DaysOff;

                bool isUnableToConnectToServer = (check.LicenceType == LicenceType.NotLoad) || (check.LicenceType == LicenceType.Error);
                bool trialLicenseIsExpired = check.LicenceType == LicenceType.TrialDaysOff;

                if (isUnableToConnectToServer) ShowServerIsUnavailableMessageBox();
                if (trialLicenseIsExpired || isUnableToConnectToServer) return false;
            }
            catch (System.Exception exception)
            {
                LogManager.WriteMessageToCustomLog("Error in GetLicenceData " + exception.ToString());
                return false;
            }
            return true;
        }

        private void ShowServerIsUnavailableMessageBox()
        {
            MessageBox.Show(Resources.SERVER_NOT_AVAILABLE, Resources.APPLICATION_NAME, MessageBoxButtons.OK, MessageBoxIcon.Hand);
        }

        private IEnumerable<string> LoadTFSServersSavedByTeamExplorer()
        {
            List<string> servers = new List<string>();

            const string name = @"Software\Microsoft\VisualStudio\8.0\TeamFoundation\Servers";
            try
            {
                using (RegistryKey key = Registry.CurrentUser.OpenSubKey(name, false))
                {
                    if (key == null) return servers;

                    foreach (string tfsServerNames in key.GetValueNames())
                    {
                        object tfsServerUrl = key.GetValue(tfsServerNames);
                        if (tfsServerUrl != null)
                        {
                            servers.Add(tfsServerUrl.ToString());
                        }
                    }
                }
            }
            catch (System.Exception exception)
            {
                LogManager.WriteMessageToCustomLog("LoadTFSServersSavedByTeamExplorer error: " + exception.ToString());
            }

            return servers;
        }

        private bool IsTXLookerFolder(MAPIFolder folder)
        {
            return folder.FullFolderPath.Contains(@"\\" + Resources.APPLICATION_NAME);
        }

        private void LoadConfigs(ref TeamXpandConfigs configs)
        {
            if (configs == null)
            {
                configs = new TeamXpandConfigs();
            }
            if (File.Exists(this.ConfigFilePath))
            {
                using (StreamReader reader = new StreamReader(this.ConfigFilePath))
                {
                    XmlSerializer serializer = new XmlSerializer(configs.GetType());
                    configs = (TeamXpandConfigs)serializer.Deserialize(reader);
                }
            }
            else
            {
                IEnumerable<string> teamExplorerServers = this.LoadTFSServersSavedByTeamExplorer();
                if (teamExplorerServers != null)
                {
                    foreach (string str in teamExplorerServers)
                    {
                        ServerConfigItem item = new ServerConfigItem();
                        item.ServerURI = str.EndsWith("/") ? str : (str + "/");
                        item.IsTeamExplorer = true;
                        try
                        {
                            IEnumerable<ProjectInfo> projects = TeamFoundationDataProvider.GetProjects(str);
                            if (projects != null)
                            {
                                foreach (ProjectInfo info in projects)
                                {
                                    item.ProjectURIs.Add(info.Uri);
                                }
                            }
                        }
                        catch (System.Exception exception)
                        {
                            MessageBox.Show(exception.ToString(), Resources.APPLICATION_NAME, MessageBoxButtons.OK, MessageBoxIcon.Hand);
                        }
                        configs.Servers.Add(item);
                    }
                }
            }
        }

        private void LoadOlStore()
        {
            try
            {
                try
                {
                    bool lookerFolderExists = false;
                    foreach (MAPIFolder folder in Outlook.Session.Folders)
                        if (folder.Store.DisplayName == Resources.APPLICATION_NAME)
                        {
                            lookerFolderExists = true;
                            this.storeID = folder.Store.StoreID;
                            break;
                        }
                    if (!lookerFolderExists)
                    {
                        Outlook.Session.Stores.StoreAdd += Stores_StoreAdd;
                        Outlook.Session.AddStoreEx(ApplicationGlobal.OlDataProvider.OutlookStorePath, OlStoreType.olStoreDefault);
                    }
                    LogManager.WriteMessageToCustomLog("Store created");
                }
                catch (System.Exception exception1)
                {
                    LogManager.WriteMessageToCustomLog("Error in store creation: " + exception1.ToString());
                }
                foreach (MAPIFolder folder in Outlook.Session.Folders)
                {
                    if (folder.StoreID == this.storeID)
                    {
                        folder.Name = Resources.APPLICATION_NAME;
                        ApplicationGlobal.OlDataProvider.AppMAPIFolderRoot = folder;
                        break;
                    }
                }
                this.RemoveDeletedItemsFolder();
            }
            catch (System.Exception exception2)
            {
                LogManager.WriteMessageToCustomLog("Error in LoadOlStore(Connect.cs) " + exception2);
            }
        }

        OutlookUIFactory uiFactory;

        private void LoadOlUIControls(Explorer explorer)
        {
            try
            {
                CommandBar bar = explorer.CommandBars["Menu Bar"];
                bar.Visible = true;
                foreach (CommandBarControl control in bar.Controls)
                {
                    if (control.Caption == Resources.UI_APPLICATION_MENU_ITEM_NAME)
                    {
                        control.Visible = true;
                        return;
                    }
                }

                CommandBarPopup ctrl = uiFactory.CreateParentButton(bar, Resources.UI_APPLICATION_MENU_ITEM_NAME);
                RegisterControl(UIControlsType.RootMenuItem, ctrl);


                RegisterControl(
                    UIControlsType.Options,
                    uiFactory.CreateChildButtonForPopup(ctrl, Resources.connect_to_tfs, Resources.connect_to_tfs_black, Resources.UI_OPTIONS_CAPTION, OnOptionsClick)
                );

                RegisterControl(
                    UIControlsType.About,
                    uiFactory.CreateChildButtonForPopup(ctrl, Resources.about, Resources.about_black, Resources.UI_ABOUT_CAPTION, OnAboutClick)
                );
                
                if ((ApplicationGlobal.LicenceDataProvider.LicenceType != LicenceType.Full) && (ApplicationGlobal.LicenceDataProvider.LicenceType != LicenceType.FullDaysOff))
                {
                    RegisterControl(
                        UIControlsType.Registration,
                        uiFactory.CreateChildButtonForPopup(ctrl, Resources.purchase, Resources.purchase_black, Resources.UI_REGISTRATION, OnRegistrationClick)
                    );
                }

                ApplicationGlobal.OlDataProvider.AppMenuItemRoot = ctrl;
            }
            catch (Exception exception)
            {
                LogManager.WriteMessageToCustomLog("Error in controls creating process: " + exception.ToString());
            }
        }

        private void ProtectOperation(UnsafeAction action)
        {
            try
            {
                action();
            }
            catch (Exception unhandledException)
            {
                LogManager.WriteMessageToCustomLog(unhandledException);
            }
        }

        private void NotifyWindowDialog()
        {
            
            LogManager.WriteMessageToCustomLog("Licence Type: " + ApplicationGlobal.LicenceDataProvider.LicenceType.ToString());

            bool fullLicenseIsExpired = ApplicationGlobal.LicenceDataProvider.LicenceType == LicenceType.FullDaysOff;
            bool trialLicenseIsExpired = ApplicationGlobal.LicenceDataProvider.LicenceType == LicenceType.TrialDaysOff;
            bool trialLicenseIsInUse = ApplicationGlobal.LicenceDataProvider.LicenceType == LicenceType.Trial;
            bool everythingIsOK = !fullLicenseIsExpired && !trialLicenseIsInUse && !trialLicenseIsExpired;

            if(everythingIsOK)
            {
                LogManager.WriteMessageToCustomLog("NOT LOAD NOTIFY WINDOW Licence Type is " + ApplicationGlobal.LicenceDataProvider.LicenceType);
                return;
            }

            string dialogMessage = Resources.LICENCE_VERSION_EXPIRED;
            if (trialLicenseIsExpired)
            {
                dialogMessage = Resources.TRIAL_VERSION_EXPIRED;
            }
            if (trialLicenseIsInUse)
            {
                dialogMessage = Resources.TRIAL_VERSION;
            }

            Registration_Dialog dialog = new Registration_Dialog(dialogMessage, 0, Outlook, false, true);
            dialog.CreateEmail += RegistrationDialog_CreateEmail;
            dialog.ShowDialog();
        }

        void RegistrationDialog_CreateEmail(object sender, EventArgs e)
        {
            ProtectOperation(
                CreateLicenceEMail
            );
        }

        public Application Outlook
        {
            get { return ApplicationGlobal.OlDataProvider.Application; }
            set { ApplicationGlobal.OlDataProvider.Application = value; }
        }

        private object addInInstance;

        public object AddIn
        {
            get { return addInInstance; }
            set { addInInstance = value; }
        }

        #region Implementation of IDTExtensibility2

        public void OnConnection(object Application, ext_ConnectMode ConnectMode, object AddInInst, ref Array custom)
        {
            LogManager.WriteMessageToCustomLog("Startup begin");
            
            Outlook = (Application)Application;
            AddIn = AddInInst;

            EventListener.Shutdown shutdownHandlerDelegate = this.OnExit;
            EventListener listener = new EventListener(Outlook, shutdownHandlerDelegate);

            // Do no start new session - just use the one is initialized
            m_Session = Outlook.Session;

            LogManager.WriteMessageToCustomLog("ApplicationGlobal.OlDataProvider.Application object is set");
            ApplicationGlobal.OlDataProvider.Session = m_Session;

            // create UI Factory as well
            uiFactory = new OutlookUIFactory();
        }

        public void OnStartupComplete(ref Array custom)
        {
            try
            {
                Hwnd = DllImportCommon.GetForegroundWindow();

                m_expl = Outlook.Explorers;
                m_expl.NewExplorer += OnNewExplorer;

                StartUpConfigsLoading(this);
                if (GetLicenceData())
                {
                    StartUpLoadOlUIControls(this);
                    StartUpExtentions(this);
                }

                StartUpUpdate(this);
            }
            catch (Exception exception)
            {
                LogManager.WriteMessageToCustomLog("Error in OnConnection " + exception.ToString());
            }

            SetFolderMenu();
            LogManager.WriteMessageToCustomLog("LicenceType " + ApplicationGlobal.LicenceDataProvider.LicenceType.ToString());

            NotifyWindowDialog();
            LogManager.WriteMessageToCustomLog("Folder tree creation complete and folder menu is set");

        }



        public void OnAddInsUpdate(ref Array custom)
        {
        }

        public void OnBeginShutdown(ref Array custom)
        {
        }

        public void OnDisconnection(ext_DisconnectMode RemoveMode, ref Array custom)
        {
        }

        #endregion


        #region Implementation of IRibbonExtensibility interface

        public string GetCustomUI(string RibbonID)
        {
            switch (RibbonID)
            {
                case "Microsoft.Outlook.MeetingRequest.Read":
                case "Microsoft.Outlook.Mail.Read":
                    return Resources.customUIMailRead;

                case "Microsoft.Outlook.MeetingRequest.Send":
                case "Microsoft.Outlook.Mail.Compose":
                    return Resources.customUIMailCompose;

                case "Microsoft.Outlook.Appointment":
                    return Resources.customUIAppointment;

                case "Microsoft.Outlook.Task":
                    return Resources.customUITask;
            }
            return string.Empty;
        }

        /// <summary>
        /// This will be called from Ribbon by custom UI elements.
        /// Usually to create Task...
        /// </summary>
        /// <param name="control">The control initiated a call</param>
        public void OnCreateWorkItemClick(IRibbonControl control)
        {
            eventSink.Publish(EventNames.CreateWorkItem);
        }

        #endregion

        private void OnExit()
        {
            try
            {
                if (this.m_extentionManager != null)
                {
                    this.m_extentionManager.Shutdown();
                    this.m_extentionManager = null;
                }
            }
            catch (System.Exception exception)
            {
                LogManager.WriteMessageToCustomLog("Error in OnExit " + exception.ToString());
            }
        }

        private void OnNewExplorer(Explorer explorer)
        {
            LoadOlUIControls(explorer);
            FolderProtectionActivate();
        }

        #region Main menu handlers

        private bool OnOptionsClick()
        {
            TFServerConnectionForm form = new TFServerConnectionForm(this.m_configs.Servers);
            if (form.ShowDialog() == DialogResult.OK)
            {
                this.m_configs.Servers.Clear();
                this.m_configs.Servers.AddRange(form.ServerConfigItems);
                this.SaveConfigs(this.m_configs);
                Cursor current = Cursor.Current;
                Cursor.Current = Cursors.WaitCursor;
                this.Update();
                Cursor.Current = current;
            }

            return false;
        }

        private bool OnRegistrationClick()
        {
            if (Registration() && (ApplicationGlobal.WorkItemTrakingExtention == null))
            {
                StartUpExtentions(this);
            }

            return false;
        }

        private bool OnAboutClick()
        {
            new AboutBox().ShowDialog();

            return false;
        }

        #endregion

        private void protectedFolder_BeforeFolderMove(MAPIFolder MoveTo, ref bool Cancel)
        {
            Cancel = true;
        }

        private void RegisterControl(UIControlsType ctrlType, CommandBarControl ctrl)
        {
            if (ctrl == null)
            {
                throw new NullReferenceException();
            }
            lock (this.m_controlsMAP)
            {
                if (!this.m_controlsMAP.ContainsKey(ctrlType))
                {
                    this.m_controlsMAP.Add(ctrlType, new List<CommandBarControl>());
                }
                this.m_controlsMAP[ctrlType].Add(ctrl);
            }
        }

        private bool Registration()
        {
            try
            {
                RegistrationForm form = new RegistrationForm(Outlook);
                form.Show();
                return form.Registrate;
            }
            catch (Exception exception)
            {
                MessageBox.Show(exception.Message, Resources.STRING_REGISTRATION_FAILED, MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                return false;
            }
        }

        private void RemoveDeletedItemsFolder()
        {
            try
            {
                Store store = m_Session.Stores[Resources.APPLICATION_NAME];
                Folder folder = GetFolderFromStore(store, "Deleted Items");
                if (folder != null)
                {
                    folder.Delete();
                }
            }
            catch
            {
            }
        }

        public Folder GetFolderFromStore(Store store, string folderName)
        {
            foreach (Folder folder in store.GetRootFolder().Folders)
            {
                if (String.CompareOrdinal(folder.Name, folderName) == 0)
                {
                    return folder;
                }
            }

            return null;
        }

        public void Ribbon_OnLoad(IRibbonUI ribbon)
        {
            Debug.WriteLine("Ribbon_OnLoad");
            ApplicationGlobal.RibbonUI = ribbon;
        }

        private void SaveConfigs(TeamXpandConfigs configs)
        {
            if (configs == null)
            {
                throw new ArgumentException();
            }
            using (StreamWriter writer = new StreamWriter(this.ConfigFilePath, false))
            {
                new XmlSerializer(configs.GetType()).Serialize((TextWriter)writer, configs);
            }
        }

        private void SetFolderMenu()
        {
            Outlook.FolderContextMenuDisplay += Application_FolderContextMenuDisplay;
        }

        private void StartUpConfigsLoading(object state)
        {
            try
            {
                this.LoadConfigs(ref this.m_configs);
                this.LoadOlStore();
                LogManager.WriteMessageToCustomLog("Configs and store loaded");
            }
            catch (Exception exception)
            {
                LogManager.WriteMessageToCustomLog("Error in StartUpConfigsLoading(Connect.cs) " + exception.ToString());
            }
        }

        private void StartUpExtentions(object state)
        {
            try
            {
                this.m_extProvider = new ExtentionDataProvider(ApplicationGlobal.TeamFoundationDataProvider, ApplicationGlobal.OlDataProvider, this.m_Session);
                this.m_extentionManager = new ExtentionManager(this.m_extProvider);
                LogManager.WriteMessageToCustomLog("Extentions startup begin");
                this.m_extentionManager.StartUp(eventSink);
                this.FolderProtectionActivate();
                LogManager.WriteMessageToCustomLog("Folder Protection Activated");
            }
            catch (Exception exception)
            {
                LogManager.WriteMessageToCustomLog("Error in StartUpExtentions(Connect.cs) " + exception.ToString());
            }
        }

        private void StartUpLoadOlUIControls(object state)
        {
            LogManager.WriteMessageToCustomLog("Begin load controls");
            this.LoadOlUIControls(Outlook.ActiveExplorer());
            LogManager.WriteMessageToCustomLog("Controls loaded");
        }

        private void StartUpUpdate(object state)
        {
            this.SetFolderMenu();
            this.Update();
            LogManager.WriteMessageToCustomLog("Folder tree creation complete and folder menu is set");
        }

        private void Stores_StoreAdd(Store Store)
        {
            this.storeID = Store.StoreID;
            try
            {
                foreach (MAPIFolder folder in Outlook.Session.Folders)
                {
                    if (folder.StoreID == this.storeID)
                    {
                        folder.Name = Resources.APPLICATION_NAME;
                        ApplicationGlobal.OlDataProvider.AppMAPIFolderRoot = folder;
                        break;
                    }
                }
                Outlook.Session.Stores.StoreAdd -= Stores_StoreAdd;
            }
            catch
            {
            }
        }

        private void UnEraseFolders_BeforeFolderSwitch(object NewFolder, ref bool Cancel)
        {
            try
            {
                if (this.IsTXLookerFolder((MAPIFolder)NewFolder))
                {
                    this.protectedFolder = (Folder)NewFolder;
                    this.protectedFolder.BeforeFolderMove += protectedFolder_BeforeFolderMove;
                }
            }
            catch (Exception exception)
            {
                LogManager.WriteMessageToCustomLog("Folder protecion isn't activated: " + exception.ToString());
            }
        }

        private void Unload()
        {
            ApplicationGlobal.TeamFoundationDataProvider.RemoveServers(ApplicationGlobal.TeamFoundationDataProvider.Servers);
        }

        private void UnloadControls()
        {
            CommandBar bar = Outlook.ActiveExplorer().CommandBars["Menu Bar"];
            if (bar != null)
            {
                CommandBarControl control = bar.Controls[Resources.UI_APPLICATION_MENU_ITEM_NAME];
                if (control != null)
                {
                    control.Delete(Missing.Value);
                }
            }
            lock (this.m_controlsMAP)
            {
                this.m_controlsMAP.Clear();
            }
        }

        private void Update()
        {
            try
            {
                List<Uri> uries = new List<Uri>();
                TeamFoundationDataProvider teamFoundationDataProvider = ApplicationGlobal.TeamFoundationDataProvider;
                foreach (ServerProvider provider2 in teamFoundationDataProvider.Servers)
                {
                    uries.Add(provider2.ServerUri);
                }
                List<ServerConfigItem> items = new List<ServerConfigItem>();
                foreach (ServerConfigItem item in this.m_configs.Servers)
                {
                    ServerProvider serverProvider = teamFoundationDataProvider.GetServerProvider(item.ServerURI);
                    if (serverProvider == null)
                    {
                        items.Add(item);
                    }
                    else
                    {
                        uries.Remove(serverProvider.ServerUri);
                        serverProvider.SetProcessProjectList(item.ProjectURIs);
                        serverProvider.Update();
                    }
                }
                if (uries.Count != 0)
                {
                    teamFoundationDataProvider.RemoveServers(uries);
                }
                if (items.Count != 0)
                {
                    // Improves Outlook loading performance, but needs additional testing
                    //teamFoundationDataProvider.AddServers(items);
                    Folder appFolder = (Folder)ApplicationGlobal.OlDataProvider.AppMAPIFolderRoot;
                    string folderName = appFolder.Name;
                    
                    ApplicationGlobal.OlDataProvider.AppMAPIFolderRoot.Name = string.Format("{0}{1}",
                                                                                            folderName,
                                                                                            Resources.STRING_LOADING);
                    teamFoundationDataProvider.AddServersAsync(items, AddServersCompleted);
                    //-----------------------------------------------------------------
                }
            }
            catch (Exception exception)
            {
                string folderName = ApplicationGlobal.OlDataProvider.AppMAPIFolderRoot.Name;
                folderName = folderName.Replace(Resources.STRING_LOADING, "");
                ApplicationGlobal.OlDataProvider.AppMAPIFolderRoot.Name = folderName;
                LogManager.WriteMessageToCustomLog("Error in updating process: " + exception.ToString());
            }
        }

        private void AddServersCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            string folderName = ApplicationGlobal.OlDataProvider.AppMAPIFolderRoot.Name;
            folderName = folderName.Replace(Resources.STRING_LOADING, "");
            ApplicationGlobal.OlDataProvider.AppMAPIFolderRoot.Name = folderName;
        }

        // Properties
        private string ConfigFilePath
        {
            get
            {
                if (this.m_configFilePath == null)
                {
                    this.m_configFilePath = Path.Combine(ApplicationGlobal.OlDataProvider.AppStoreFolderPath, "TeamXpandConfigs.xml");
                }
                return this.m_configFilePath;
            }
        }

        public static IntPtr Hwnd
        {
            get
            {
                return m_hWnd;
            }
            set
            {
                m_hWnd = value;
            }
        }
    }
}